#!/usr/bin/env pythonx
"""Implements methods for common input/output operations"""
import scipy
import librosa
import logging

from pydub import AudioSegment

COMMA_DELIMITER = ','

MODE_WRITE = 'w'
MODE_APPEND = 'a'
MODE_WRITE_BINARY = 'wb'

logger = logging.getLogger(__name__)


def read_audio(filepaths, library='librosa', **kwargs):
    instances_raw, sampling_rates = [], []
    if library == 'librosa':
        for filepath in filepaths:
            try:
                aud_time_series, sr = librosa.load(filepath, **kwargs)
                instances_raw.append(aud_time_series)
                sampling_rates.append(sr)
            except Exception as e:
                instances_raw.append(None)
                sampling_rates.append(None)
                logger.error('Unable to read file {}. {}'.format(filepath, str(e)))

        return instances_raw, sampling_rates

    elif library == 'scipy':
        for filepath in filepaths:
            try:
                sr, aud_time_series = scipy.io.wavfile.read(filepath)
                instances_raw.append(aud_time_series)
                sampling_rates.append(sr)
            except Exception as e:
                instances_raw.append(None)
                sampling_rates.append(None)
                logger.error('Unable to read file {}. {}'.format(filepath, str(e)))

        return instances_raw, sampling_rates

    elif library == 'pydub':
        audio_objects = []
        for filepath in filepaths:
            try:
                audio_obj = AudioSegment.from_file(filepath)
            except Exception as e:
                logger.error('Unable to read file {}. {}'.format(filepath, str(e)))
                audio_objects.append(None)
            else:
                audio_objects.append(audio_obj)

        return audio_objects, None