#!/usr/bin/env pythonx
"""Implements methods for common input/output operations"""
import os
import json
import pickle
import logging

import numpy as np
import pandas as pd

COMMA_DELIMITER = ','

MODE_WRITE = 'w'
MODE_APPEND = 'a'
MODE_WRITE_BINARY = 'wb'

logger = logging.getLogger(__name__)


def read_file(filepath, delimiter=COMMA_DELIMITER):
    """
    Function to read file based on extension of the file format to be read
    :param filepath: File location and name to be read
    :param delimiter: delimiter for reading csv
    :return: Data of the file.
    """
    extension = os.path.splitext(filepath)[1]
    try:
        if extension == '.json':
            with open(filepath, 'r') as filepath:
                data = json.load(filepath)

        elif extension in ['.tsv', '.txt', '.csv']:
            try:
                data = pd.read_csv(filepath, sep=delimiter, header=0)
            except Exception:
                data = pd.read_csv(
                    filepath, sep=delimiter, encoding='latin'
                )

        elif extension == '.pkl':
            with open(filepath, 'rb') as filepath:
                data = pickle.load(filepath)

        elif extension == '.npy':
            data = np.load(filepath, allow_pickle=True)

        elif extension == '.hdf5':
            from keras.models import load_model
            data = load_model(filepath)

        else:
            raise Exception(f'Invalid extension: {extension}')

    except Exception as e:
        logger.error('Unable to read file {}. {}'.format(filepath, str(e)))
        raise

    return data