import logging
import python_speech_features as ps

from JarvisML.common.read.read_audio import read_audio

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class ExtractDeltaFeatures:
    def run(self, instances, base='logfbank', delta_N=2, **kwargs):
        """Standalone function to extract differentiation and double differentiation over base features

        :param instances: raw loaded audio
        :param base: base features can be 'logfbank' or 'mfcc'
        :param kwargs: {'base': 'mfcc', 'read_kwargs': {}, 'extractor_kwargs': {}}
        :return: extracted features
        """
        # Read data
        read_kwargs = kwargs.get('read_kwargs', {})
        instances_raw, sampling_rates = read_audio(filepaths=instances, **read_kwargs)
        extractor_kwargs = kwargs.get('extractor_kwargs', {})

        delta_features = []

        def get_delta_features(base_process_callable):
            for instance, sr in zip(instances_raw, sampling_rates):
                base_features = base_process_callable(instance, sr, **extractor_kwargs)
                try:
                    delta_differential = ps.delta(base_features, delta_N)
                    delta_acceleration = ps.delta(delta_differential, delta_N)

                    feature_vector = [base_features, delta_differential, delta_acceleration]
                    delta_features.append(feature_vector)
                except Exception as e:
                    delta_features.append(None)
                    logger.error('Unable to extract feature', e)

            return delta_features

        assert base == 'logfbank' or base == 'mfcc', "base '{}' not supported".format(base)

        if base == 'logfbank':
            base_process_callable = ps.logfbank
        elif base == 'mfcc':
            base_process_callable = ps.mfcc

        delta_features = get_delta_features(base_process_callable)

        return delta_features