import logging
import librosa

from JarvisML.common.read.read_audio import read_audio
from copy import deepcopy

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class ExtractMagnitudeSpectrograms:
    def run(self, instances, read_kwargs=None, extractor_kwargs=None, **kwargs):
        spectrograms = []

        # Read data
        read_kwargs = read_kwargs or {}
        instances_raw, sampling_rates = read_audio(filepaths=instances, **read_kwargs)

        # Extract features
        extractor_kwargs = extractor_kwargs or {}
        final_extractor_kwargs = deepcopy(extractor_kwargs)

        # Assuming the sampling rates of all audios will be the same, else move this in the loop below
        if 'n_fft' in extractor_kwargs:
            final_extractor_kwargs['n_fft'] = extractor_kwargs['n_fft']
        if 'win_length' in extractor_kwargs:
            final_extractor_kwargs['win_length'] = int(extractor_kwargs['win_length'] * sampling_rates[0])
        if 'hop_length' in extractor_kwargs:
            final_extractor_kwargs['hop_length'] = int(extractor_kwargs['hop_length'] * sampling_rates[0])

        for instance in instances_raw:
            try:
                stft = librosa.stft(instance, **final_extractor_kwargs)
                # To ignore the phase part returned by stft
                spectrogram = librosa.magphase(stft)[0]
                spectrograms.append(spectrogram)
            except Exception as e:
                spectrograms.append(None)
                logger.error('Unable to extract feature', e)

        return spectrograms
