import logging
import librosa

from JarvisML.common.read.read_audio import read_audio
from copy import deepcopy

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class ExtractMelSpectrograms:
    def run(self, instances, **kwargs):
        mel_spectrograms = []

        # Read data
        read_kwargs = kwargs.get('read_kwargs', {})
        instances_raw, sampling_rates = read_audio(filepaths=instances, **read_kwargs)

        # Extract features
        extractor_kwargs = kwargs.get('extractor_kwargs', {})
        final_extractor_kwargs = deepcopy(extractor_kwargs)

        # Assuming the sampling rates of all audios will be the same, else move this in the loop below
        if 'n_fft' in extractor_kwargs:
            final_extractor_kwargs['n_fft'] = int(extractor_kwargs['n_fft'] * sampling_rates[0])
        if 'hop_length' in extractor_kwargs:
            final_extractor_kwargs['hop_length'] = int(extractor_kwargs['hop_length'] * sampling_rates[0])

        for instance, sr in zip(instances_raw, sampling_rates):
            try:
                mel_spect = librosa.feature.melspectrogram(y=instance, sr=sr, **final_extractor_kwargs)
                mel_spectrograms.append(mel_spect.reshape((mel_spect.shape[1], mel_spect.shape[0])))

            except Exception as e:
                mel_spectrograms.append(None)
                logger.error('Unable to extract feature', e)

        return mel_spectrograms
