import logging
import numpy as np

from JarvisML.common.read.read_audio import read_audio
from utils.logger import log_entry_exit

logger = logging.getLogger(__name__)


class GetAudioFromStartAndEnd:

    @log_entry_exit(log_time=False)
    def run(self, data, duration_to_fetch=22, read_kwargs=None):
        """
        Get audio signal of duration "duration_to_fetch" from start and end of the call, and concatenate them together.
        :param data: Audio file paths
        :param duration_to_fetch: Time interval that is required from both start and end of the recording [in seconds].
        :param read_kwargs: kwargs meant for reading audio files
        :param kwargs:
        :return: processed_data: List of lists of the form [[aud1, sample_rate1], [aud2, sample_rate2] ...]
        """
        read_kwargs = read_kwargs or {}

        raw_audios, sampling_rates = read_audio(data, **read_kwargs)
        processed_data = []
        for file_path, raw_audio, sampling_rate in zip(data, raw_audios, sampling_rates):
            try:
                samples_to_fetch = duration_to_fetch * sampling_rate
                audio_from_start = raw_audio[:samples_to_fetch]
                audio_from_end = raw_audio[-1 * samples_to_fetch:] if len(raw_audio) >= samples_to_fetch \
                    else np.array([])
                processed_audio = np.concatenate((audio_from_start, audio_from_end))
                processed_data_unit = (processed_audio, sampling_rate)
            except TypeError:
                # Exception would be raised if either of raw_audio or sampling_rate is None
                logger.error(
                    'Audio received {file_path} by GetAudioFromStartAndEnd was corrupt.'.format(file_path=file_path)
                )
                processed_data_unit = (None, None)

            processed_data.append(processed_data_unit)

        return processed_data


