import logging
import librosa
from JarvisML.utils.logger.log_entry_exit import log_entry_exit
from pyAudioAnalysis import audioSegmentation as aS

logger = logging.getLogger(__name__)


@log_entry_exit(log_time=False)
def get_speech_times(data):
    """
    Get speech times from audio files.
    :param data: Audio file paths from which silence needs to be removed.
    """

    predictions = []

    # Iterate on all the audio_files present in the dataframe
    for _path in zip(data):

        # Load all the channels present in the audio file
        y, sr = librosa.load(_path, sr=None, mono=False)

        if len(y.shape) == 1:
            y = [y]

        speech_times_json = {}

        # Run silence removal predictions for each of the channel
        for channel in range(len(y)):
            speech_times = aS.silence_removal(y[channel], sr, 0.05, 0.025, smooth_window=1, weight=0.3, plot=False)

            # From return json in start and end time format
            speech_times_json[channel] = []
            for start_time, end_time in speech_times:
                speech_times_json[channel].append(
                    {
                        'start': round(start_time, 4),
                        'end': round(end_time, 4),
                    }
                )

        predictions.append(speech_times_json)

    return predictions