import logging
from JarvisML.utils.logger.log_entry_exit import log_entry_exit

logger = logging.getLogger(__name__)


@log_entry_exit(log_time=False)
def merge_ranges_with_small_diff(ranges_list, speech_time_margin):
    """
    Smooth speech time ranges with small margin. The operations performed here are:
    1. For a speech range represented by [st_time, en_time] => [st_time-speech_time_margin, en_time+speech_time_margin]
    2. If there are 2 speech ranges after the above step [st_time_1, en_time_1], [st_time_2, en_time_2] such that
    en_time_1 >= st_time_2 merge the two ranges to form [st_time_1, en_time_2]
    :param ranges_list: List of lists representing speech times [[st_time_1, en_time_1], [st_time_2, en_time_2] ... ]
    :param speech_time_margin: The margin of error to add to speech times from both left and right side
    :return: final_range_array: Final smoothed ranges following the description provided
    """

    final_ranges_list = []
    for range_dp in ranges_list:
        if len(final_ranges_list) == 0:
            range_dp = [max(0, range_dp[0]-speech_time_margin), range_dp[1] + speech_time_margin]
            final_ranges_list.append(range_dp)
        else:
            range_dp = [range_dp[0] - speech_time_margin, range_dp[1] + speech_time_margin]
            last_range_dp = final_ranges_list[-1]
            if last_range_dp[1] >= range_dp[0]:
                final_ranges_list[-1] = [last_range_dp[0], range_dp[1]]
            else:
                final_ranges_list.append(range_dp)

    return final_ranges_list