import boto3
import os
from tqdm import tqdm
tqdm.pandas()


def fetch_audio_file_from_s3_bucket(bucket_name):
    # Initialize an S3 client
    s3 = boto3.client('s3')
    # List objects in the S3 bucket
    response = s3.list_objects_v2(Bucket=bucket_name, Prefix='folder-name')
    local_directory = 'audio_data'

    # Iterate through the objects in the bucket
    for obj in response.get('Contents', []):
        # Extract the key (object key)
        object_key = obj['Key']
        try:
            # Download the WAV file
            wav_data = s3.get_object(Bucket=bucket_name, Key=object_key)
            # Save the WAV data to a local file
            local_file_path = os.path.join(local_directory, os.path.basename(object_key))
            with open(local_file_path, 'wb') as local_file:
                local_file.write(wav_data['Body'].read())
            print(f"Downloaded {object_key} to {local_file_path}")
        except Exception as e:
            print(f"Failed to fetch WAV: {object_key} - {str(e)}")
